==> xcdsutils-1.0.48.tgz: package/ebike/XCYuntuUtil.js <==
const rp = require('request-promise');

/**
 * 在模块初始化yuntu配置时调用
 * 确认yuntu.deviceTableid不为空，否则创建并写入
 * 除非创建不成功则仍为空，此时可能的问题在于yuntu上已存在表但config中未记录
 * @param {Object} yuntu 在初始化config时从config.json中读出的数据
 */
async function deviceTableidEnsureInit(yuntu) {
    if(yuntu.deviceTableid === undefined) {
        //config.json中未配置deviceTableid，则主动创建gFenceTable
        const tableid = await _tableCreate(yuntu, "deviceTable");
        if(tableid !== undefined) {
            //创建成功则写入yuntu.deviceTableid
            yuntu.deviceTableid = tableid;
        }
    }

    return;
}

/**
 * 在模块初始化yuntu配置时调用
 * 确认yuntu.gFenceTableid不为空，否则创建并写入
 * 除非创建不成功则仍为空，此时可能的问题在于yuntu上已存在表但config中未记录
 * @param {Object} yuntu 在初始化config时从config.json中读出的数据
 */
async function gFenceTableidEnsureInit(yuntu) {
    if(yuntu.gFenceServiceTableid === undefined) {
        const tableid = await _tableCreate(yuntu, "gFenceServiceTable");
        if(tableid !== undefined) {
            yuntu.gFenceServiceTableid = tableid;
        }
    }

    if(yuntu.gFenceUsableTableid === undefined) {
        const tableid = await _tableCreate(yuntu, "gFenceUsableTable");
        if(tableid !== undefined) {
            yuntu.gFenceUsableTableid = tableid;
        }
    }

    return;
}

/**
 * 在更新设备GPS或者电子围栏中心点GPS时调用
 * 根据name查找，未找到则创建，找到一个则更新，找到多个则说明yuntu中数据出错，则全部删除后再创建
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} name     电子围栏在yuntu表中的名称，暂定为`${gFenceType}_${gFenceId}`
 * @param {string} location 电子围栏在yuntu表中的中心点GPS，暂定为`${lng},${lat}`
 */
async function DataUpdate(yuntu, tableid, name, location) {
    const [dataCount, dataIds] = await _dataFind(yuntu, tableid, name);
    if(dataCount === 0) {
        _dataCreate(yuntu, tableid, name, location);
    } else if(dataCount === 1) {
        _dataUpdate(yuntu, tableid, dataIds[0], name, location);
    } else if(dataCount > 1) {
        const dataIdsStr = dataIds.join(",");
        const success = await _dataDelete(yuntu, tableid, dataIdsStr);//考虑到可能有多个相同gFence同时删除，则只在删除数目正确的时候才重新创建
        if(success === dataCount) {
            _dataCreate(yuntu, tableid, name, location);
        }
    }

    return;
}

/**
 * 在查找附近电子围栏时调用
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} location 查找中心点的GPS，暂定为`${lng},${lat}`
 * @param {number} radius   查找范围的半径，取值范围[0,50000]，单位：米
 * @returns {Object[]}      查找到的结果
 */
async function DataSearchAround(yuntu, tableid, location, radius) {
    const datas = await _dataSearchAround(yuntu, tableid, location, radius);
    return datas;
}

/**
 * 内部函数
 * 创建表并返回tableid
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {string} name     创建的表的表名
 * @returns {number}        创建成功的tableid
 */
async function _tableCreate(yuntu, name) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/table/create',
        formData: {
            key: yuntu.key,
            name: name,
        },
    };
    const rsp = await rp(options);
    
    console.info("_tableCreate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let tableid;
    if(rspJson.status === 1) {
        //create table succeed
        //console.info("_tableCreate tableid:", rspJson.tableid);
        tableid = rspJson.tableid;
    } else {
        //create table failed
    }

    return tableid;
}

/**
 * 内部函数
 * 新建数据
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  yuntu中的表id
 * @param {string} name     创建的数据的名字
 * @param {string} name     创建的数据的GPS
 * @returns {}              无
 */
async function _dataCreate(yuntu, tableid, name, location) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/create',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            loctype: 1,
            data: JSON.stringify({
                _name: name,
                _location: location,
            })
        },
    };
    const rsp = await rp(options);

    console.info("_dataCreate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    if(rspJson.status === 1) {
        //_dataCreate succeed
        //console.info("_dataCreate dataid:", rspJson._id);
    } else {
        //_dataCreate failed
    }

    return;
}

/**
 * 内部函数
 * 删除数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} dataidsStr   删除的数据id的字符串集合
 * @returns {number}            删除成功的数目
 */
async function _dataDelete(yuntu, tableid, dataidsStr) {
    console.info("_dataDelete; tableid, dataidsStr:", tableid, dataidsStr);
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/delete',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            ids: dataidsStr,
        },
    };
    const rsp = await rp(options);

    console.info("_dataDelete rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let success;
    if(rspJson.status === 1) {
        //succeed
        //console.info("_dataDelete; success, fail:", rspJson.success, rspJson.fail);
        success = rspJson.success;
    } else {
        //failed
    }

    return success;
}

/**
 * 内部函数
 * 更新数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {number} dataid       yuntu中的数据id
 * @param {string} name         更新的数据名字，一般不会主动修改数据名字
 * @param {string} location     更新的数据GPS
 * @returns {}                  无
 */
async function _dataUpdate(yuntu, tableid, dataid, name, location) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/update',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            loctype: 1,
            data: JSON.stringify({
                _id: dataid,
                _name: name,
                _location: location,
            })
        },
    };
    const rsp = await rp(options);

    console.info("_dataUpdate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    if(rspJson.status === 1) {
        //create table succeed
    } else {
        //create table failed
    }

    return;
}

/**
 * 内部函数
 * 查找数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} name         要查找的数据名字，一般要保持名字的唯一性
 * @returns {[number, number[]]}与此数据名字相同的个数，以及各个数据id
 */
async function _dataFind(yuntu, tableid, name) {
    console.info("_dataFind tableid, name:", tableid, name);
    const options = {
        method: 'get',
        uri: yuntu.uriBase + '/datamanage/data/list',
        qs: {
            key: yuntu.key,
            tableid: tableid,
            filter: `_name:${name}`,
        },
    };
    const rsp = await rp(options);

    console.info("_dataFind rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let dataCount = rspJson.count;
    let dataIds = [];
    if(dataCount === 1) {
        //_dataFind succeed
        dataIds.push(rspJson.datas[0]._id);
    } else if(dataCount > 1) {
        console.warn("_dataFind count greater than 1!");
        for (let data of rspJson.datas) {
            dataIds.push(data._id);
        }
    }

    return [dataCount, dataIds];
}

/**
 * 内部函数
 * 搜索附近数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} location     要搜索的点的定位
 * @param {number} radius       要搜索的范围半径
 * @returns {Object[]}          与此数据名字相同的个数，以及各个数据id
 */
async function _dataSearchAround(yuntu, tableid, location, radius) {
    const options = {
        method: 'get',
        uri: yuntu.uriBase + '/datasearch/around',
        qs: {
            key: yuntu.key,
            tableid: tableid,
            center: location,
            radius: radius,
        },
    };
    const rsp = await rp(options);

    console.debug("_dataSearchAround rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let datas;
    if(rspJson.status === 1) {
        //create table succeed
        console.info("_dataSearchAround count:", rspJson.count);
        datas = rspJson.datas;
    } else {
        //create table failed
    }

    return datas;
}

module.exports = {
    deviceTableidEnsureInit,
    gFenceTableidEnsureInit,

    DataUpdate,
    DataSearchAround,
}

==> xcdsutils-1.0.50.tgz: package/ebike/XCYuntuUtil.js <==
const rp = require('request-promise');

/**
 * 在模块初始化yuntu配置时调用
 * 确认yuntu.deviceTableid不为空，否则创建并写入
 * 除非创建不成功则仍为空，此时可能的问题在于yuntu上已存在表但config中未记录
 * @param {Object}  yuntu 在初始化config时从config.json中读出的数据
 * @returns {}      无
 */
async function deviceTableidEnsureInit(yuntu) {
    if(yuntu.deviceTableid === undefined) {
        //config.json中未配置deviceTableid，则主动创建gFenceTable
        const tableid = await _tableCreate(yuntu, "deviceTable");
        if(tableid !== undefined) {
            //创建成功则写入yuntu.deviceTableid
            yuntu.deviceTableid = tableid;
        }
    }

    return;
}

/**
 * 在模块初始化yuntu配置时调用
 * 确认yuntu.gFenceTableid不为空，否则创建并写入
 * 除非创建不成功则仍为空，此时可能的问题在于yuntu上已存在表但config中未记录
 * @param {Object} yuntu 在初始化config时从config.json中读出的数据
 * @returns {}      无
 */
async function gFenceTableidEnsureInit(yuntu) {
    if(yuntu.gFenceServiceTableid === undefined) {
        const tableid = await _tableCreate(yuntu, "gFenceServiceTable");
        if(tableid !== undefined) {
            yuntu.gFenceServiceTableid = tableid;
        }
    }

    if(yuntu.gFenceUsableTableid === undefined) {
        const tableid = await _tableCreate(yuntu, "gFenceUsableTable");
        if(tableid !== undefined) {
            yuntu.gFenceUsableTableid = tableid;
        }
    }

    return;
}

/**
 * 在增加设备GPS或者电子围栏中心点GPS时调用
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} name     电子围栏在yuntu表中的名称，暂定为`${gFenceType}_${gFenceId}`或`${imei}`
 * @param {string} location 电子围栏在yuntu表中的中心点GPS，暂定为`${lng},${lat}`
 */
async function DataCreate(yuntu, tableid, name, location) {
    return await _dataCreate(yuntu, tableid, name, location);
}

/**
 * 在删除设备GPS或者电子围栏中心点GPS时调用
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} name     电子围栏在yuntu表中的名称，暂定为`${gFenceType}_${gFenceId}`或`${imei}`
 * @returns {number}        删除成功的数目
 */
async function DataDelete(yuntu, tableid, name) {
    return await _dataDelete(yuntu, tableid, name);
}

/**
 * 在更新设备GPS或者电子围栏中心点GPS时调用
 * 根据name查找，未找到则创建，找到一个则更新，找到多个则说明yuntu中数据出错，则全部删除后再创建
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} name     电子围栏在yuntu表中的名称，暂定为`${gFenceType}_${gFenceId}`或`${imei}`
 * @param {string} location 电子围栏在yuntu表中的中心点GPS，暂定为`${lng},${lat}`
 */
async function DataUpdate(yuntu, tableid, name, location) {
    const [dataCount, dataIds] = await _dataFind(yuntu, tableid, name);
    if(dataCount === 0) {
        _dataCreate(yuntu, tableid, name, location);
    } else if(dataCount === 1) {
        _dataUpdate(yuntu, tableid, dataIds[0], name, location);
    } else if(dataCount > 1) {
        const dataIdsStr = dataIds.join(",");
        const success = await _dataDelete(yuntu, tableid, dataIdsStr);//考虑到可能有多个相同gFence同时删除，则只在删除数目正确的时候才重新创建
        if(success === dataCount) {
            _dataCreate(yuntu, tableid, name, location);
        }
    }

    return;
}

/**
 * 在查找附近电子围栏时调用
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  电子围栏种类索引到的tableid
 * @param {string} location 查找中心点的GPS，暂定为`${lng},${lat}`或`${imei}`
 * @param {number} radius   查找范围的半径，取值范围[0,50000]，单位：米
 * @returns {Object[]}      查找到的结果
 */
async function DataSearchAround(yuntu, tableid, location, radius) {
    const datas = await _dataSearchAround(yuntu, tableid, location, radius);
    return datas;
}


/**
 * 内部函数
 * 创建表并返回tableid
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {string} name     创建的表的表名
 * @returns {number}        创建成功的tableid
 */
async function _tableCreate(yuntu, name) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/table/create',
        formData: {
            key: yuntu.key,
            name: name,
        },
    };
    const rsp = await rp(options);
    
    console.info("_tableCreate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let tableid;
    if(rspJson.status === 1) {
        //create table succeed
        //console.info("_tableCreate tableid:", rspJson.tableid);
        tableid = rspJson.tableid;
    } else {
        //create table failed
    }

    return tableid;
}

/**
 * 内部函数
 * 新建数据
 * @param {Object} yuntu    在初始化config时从config.json中读出的数据
 * @param {number} tableid  yuntu中的表id
 * @param {string} name     创建的数据的名字
 * @param {string} name     创建的数据的GPS
 * @returns {number}        创建成功的数据id
 */
async function _dataCreate(yuntu, tableid, name, location) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/create',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            loctype: 1,
            data: JSON.stringify({
                _name: name,
                _location: location,
            })
        },
    };
    const rsp = await rp(options);

    console.info("_dataCreate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let dataid;
    if(rspJson.status === 1) {
        //_dataCreate succeed
        //console.info("_dataCreate dataid:", rspJson._id);
        dataid = rspJson._id;
    } else {
        //_dataCreate failed
    }

    return dataid;
}

/**
 * 内部函数
 * 删除数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} dataidsStr   删除的数据id的字符串集合
 * @returns {number}            删除成功的数目
 */
async function _dataDelete(yuntu, tableid, dataidsStr) {
    console.info("_dataDelete; tableid, dataidsStr:", tableid, dataidsStr);
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/delete',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            ids: dataidsStr,
        },
    };
    const rsp = await rp(options);

    console.info("_dataDelete rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let success;
    if(rspJson.status === 1) {
        //succeed
        //console.info("_dataDelete; success, fail:", rspJson.success, rspJson.fail);
        success = rspJson.success;
    } else {
        //failed
    }

    return success;
}

/**
 * 内部函数
 * 更新数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {number} dataid       yuntu中的数据id
 * @param {string} name         更新的数据名字，一般不会主动修改数据名字
 * @param {string} location     更新的数据GPS
 * @returns {}                  无
 */
async function _dataUpdate(yuntu, tableid, dataid, name, location) {
    const options = {
        method: 'post',
        headers: { 'content-type': 'application/x-www-form-urlencoded' },
        uri: yuntu.uriBase + '/datamanage/data/update',
        formData: {
            key: yuntu.key,
            tableid: tableid,
            loctype: 1,
            data: JSON.stringify({
                _id: dataid,
                _name: name,
                _location: location,
            })
        },
    };
    const rsp = await rp(options);

    console.info("_dataUpdate rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    if(rspJson.status === 1) {
        //create table succeed
    } else {
        //create table failed
    }

    return;
}

/**
 * 内部函数
 * 查找数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} name         要查找的数据名字，一般要保持名字的唯一性
 * @returns {[number, number[]]}与此数据名字相同的个数，以及各个数据id
 */
async function _dataFind(yuntu, tableid, name) {
    console.info("_dataFind tableid, name:", tableid, name);
    const options = {
        method: 'get',
        uri: yuntu.uriBase + '/datamanage/data/list',
        qs: {
            key: yuntu.key,
            tableid: tableid,
            filter: `_name:${name}`,
        },
    };
    const rsp = await rp(options);

    console.info("_dataFind rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let dataCount = rspJson.count;
    let dataIds = [];
    if(dataCount === 1) {
        //_dataFind succeed
        dataIds.push(rspJson.datas[0]._id);
    } else if(dataCount > 1) {
        console.warn("_dataFind count greater than 1!");
        for (let data of rspJson.datas) {
            dataIds.push(data._id);
        }
    }

    return [dataCount, dataIds];
}

/**
 * 内部函数
 * 搜索附近数据
 * @param {Object} yuntu        在初始化config时从config.json中读出的数据
 * @param {number} tableid      yuntu中的表id
 * @param {string} location     要搜索的点的定位
 * @param {number} radius       要搜索的范围半径
 * @returns {Object[]}          与此数据名字相同的个数，以及各个数据id
 */
async function _dataSearchAround(yuntu, tableid, location, radius) {
    const options = {
        method: 'get',
        uri: yuntu.uriBase + '/datasearch/around',
        qs: {
            key: yuntu.key,
            tableid: tableid,
            center: location,
            radius: radius,
        },
    };
    const rsp = await rp(options);

    console.debug("_dataSearchAround rsp:", rsp);
    const rspJson = JSON.parse(rsp);
    let datas;
    if(rspJson.status === 1) {
        //create table succeed
        console.info("_dataSearchAround count:", rspJson.count);
        datas = rspJson.datas;
    } else {
        //create table failed
    }

    return datas;
}

module.exports = {
    deviceTableidEnsureInit,
    gFenceTableidEnsureInit,

    DataCreate,
    DataDelete,
    DataUpdate,
    DataSearchAround,
}
